package praktikum;

import java.math.BigInteger;
import java.util.Random;

/** Diese Klasse implementiert einen Schlüsselgenerator für die Damgard-Hashfunktion */

public class DamgardKeyGenerator{
    DamgardKey key;

    /**
     * Schlüsselgenerator erstellen und Damgard-Schlüssel generieren.
     * @param k Länge des Schlüssels in Bit
     */
    
    public DamgardKeyGenerator(int k){
	key = new DamgardKey();
	/* p prim, p = 3 mod 4 bilden */
	
	BigInteger p = null;
	do{
	    p = new BigInteger(k / 2, 200, new Random());
	}while(p.mod(BigInteger.valueOf(4)).compareTo(BigInteger.valueOf(3)) != 0);
	/* q prim, q = 3 mod 4 bilden, p != q */
	
	BigInteger q = null;
	do{
	    q = new BigInteger(k / 2, 200, new Random());
	}while(q.mod(BigInteger.valueOf(4)).compareTo(BigInteger.valueOf(3)) != 0 
	       || q.compareTo(p) == 0);
	/* n = p*q bilden */
	
	key.n = p.multiply(q);
	/* Referenz r bilden */
	
	key.r = new BigInteger(k, new Random());
	key.r = key.r.modPow(BigInteger.valueOf(2), key.n);
    }

    /**
     * Diese Methode gibt den Schlüssel zurück.
     * @return den Damgard-Schlüssel
     */
    
    public DamgardKey getKey(){
	return key;
    }
}
