package praktikum;
 
import java.math.BigInteger; 
import algorithms.*; 
 
/**
 * Diese Klasse implementiert das (vereinfachte) Signatursystem GMR.  
 */ 
 
public class GMR{ 
 
    /**
     * Diese Methode implementiert die Signierfunktion von GMR. 
     * Die Nachricht m liegt dabei schon im BigInteger-Format vor. 
     * Vereinfachend kann hier davon ausgegangen werden, daß m < n ist. 
     *  
     * Es konnen die BigInteger-Methoden und zusätzlich die  
     * Algorithmen CRA, Legendre und SQRTMOD, und die Methoden 
     * invertMessage(), praefixfreiMessage() und QRn() verwendet werden.  
     * 
     * @param privKey der geheime Signierschlüssel 
     * @param m die Nachricht 
     * @param r die Referenz, an der signiert werden soll 
     * @return die Signatur 
     */ 
     
    public BigInteger sign(GMRPrivateKey privKey, BigInteger m, BigInteger r){ 
	try{ 
 
	    //-------------------------------------------------- 
	    /* Hier implementieren                            */
					 
	    //-------------------------------------------------- 
	     
 

 
 
	    /* nicht vergessen, die Signatur zurückzugeben */
	    
	    //------------------------------------------------------------ 
 
	}catch (Exception e){ 
	    System.out.println("GMR.sign: "+e.getMessage()); 
	    e.printStackTrace(); 
	} 
	return null; 
    } 
	 
    /** 
     * Diese Methode implementiert die Testfunktion von GMR. 
     * 
     * @param pubKey der öffentliche Schlüssel 
     * @param m die Nachricht 
     * @param sig die Signatur 
     * @return true, wenn die Signatur gültig ist, sonst false 
     */ 
     
    public boolean test(GMRPublicKey pubKey, BigInteger m, BigInteger r, BigInteger sig){ 
 
	BigInteger mp = praefixfreiMessage(m); 
	 
	BigInteger test = sig; 
 
	int l = mp.bitLength();  
	for(int i = 0; i < l; i++){ 
	    if(mp.testBit(0)){ 
		test = test.multiply(BigInteger.valueOf(2)).mod(pubKey.n); 
	    } 
	    test = test.modPow(BigInteger.valueOf(2),pubKey.n); 
	    mp = mp.divide(BigInteger.valueOf(2)); 
	} 
	 
	if (test.compareTo(pubKey.ndiv2) > 0){ 
	    test = test.subtract(pubKey.n); 
	} 
	if (test.compareTo(r) == 0){ 
	    return true; 
	} 
	return false; 
    } 
 
 
     
    /**
     * Die Methode invertMessage dreht eine Nachricht bitweise um. 
     * (das letzte Bit wird das erste, u.s.w.) 
     * 
     * @param m die umzudrehende Nachricht 
     * @return die umgedrehte Nachricht 
     */  
    
    private BigInteger invertMessage(BigInteger m){ 
	BigInteger invertM = BigInteger.ZERO; 
	BigInteger mWork = new BigInteger(m.toString()); 
	for(int i = 0; i < m.bitLength(); i++){ 
	    invertM = invertM.multiply(BigInteger.valueOf(2)); 
	    if(mWork.testBit(0)){ 
		invertM = invertM.add(BigInteger.ONE); 
	    } 
	    mWork = mWork.divide(BigInteger.valueOf(2)); 
	} 
	return invertM; 
    } 
 
     
     /** 
     * Die Methode praefixfreiMessage erzeugt aus einer Nachricht m eine präfixfrei 
     * kodierte Nachricht m, indem ihre Bitlänge angehängt wird. 
     * (um die Nachrichtenlänge resistent gegen das Invertieren der Nachricht zu machen, 
     * wird zusätzlich eine 1 angehängt) 
     *  
     * @param m die Nachicht 
     * @return die präfixfrei kodierte Nachricht 
     */ 
      
    private BigInteger praefixfreiMessage(BigInteger m){ 
	BigInteger mPFrei = m.shiftLeft(5); 
        BigInteger len = BigInteger.valueOf(mPFrei.bitLength()); 
	mPFrei = mPFrei.add(len.shiftLeft(1)); 
	mPFrei = mPFrei.add(BigInteger.ONE); 
	return mPFrei; 
    } 
 
    /** 
     * Die Methode QRn ermittelt, ob eine Zahl quadratischer Rest von 
     * n = p * q ist. 
     * 
     * @param p Primzahl p 
     * @param q Primzahl q 
     * @param x der potentielle quadratische Rest 
     * @return true, wenn x = QR(n), sonst false 
     */ 
    
    private boolean QRn(BigInteger p, BigInteger q, BigInteger x){ 
	try{ 
	    LEGENDRE lp = new LEGENDRE(x,p); 
	    LEGENDRE lq = new LEGENDRE(x,q); 
	    lp.executeLEGENDRE(); 
	    lq.executeLEGENDRE(); 
	    if(lp.getErgebnis() == 1 && lq.getErgebnis() == 1){ 
		return true; 
	    } 
	    return false; 
	}catch (Exception e){ 
	    System.out.println("GMR.QRn: "+e.getMessage()); 
	} 
	return false; 
    } 
}
