package praktikum;

import java.math.BigInteger;
import java.util.Random;
import java.util.*;

/** Diese Klasse implementiert einen Schlüsselgenerator für GMR */

public class GMRKeyGenerator{
    
    /* Fuer Internationalisierung */
    public static ResourceBundle messages;

    GMRPrivateKey privKey;
    GMRPublicKey pubKey;

    /**
     * Schlüsselgenerator erstellen und GMR-Schlüssel generieren.
     * @param k Länge des Schlüssels in Bit
     */
    
    public GMRKeyGenerator(int k){
    	this(k,new Random().nextLong());
    }
    
    public GMRKeyGenerator(int k, long seed){
    Random r = new Random(seed);
    
	
	/* Der folgende Teil ist nötig für die Internationalisierung */
	Locale currentLocale; 
	currentLocale = Locale.getDefault(); 
	messages = ResourceBundle.getBundle("praktikum.localmessages",currentLocale); 
 	
	privKey = new GMRPrivateKey();
	pubKey = new GMRPublicKey();

	System.out.println(messages.getString("GMRKeyGeneratorGenKey"));
	privKey.p = null;
	privKey.q = null;
	while((privKey.p == null) || (privKey.q == null)){
	    /* Pseudo-Primzahl der Länge k/2 bilden (mit Wkt. 1/(2^30) prim) */
	    
	    System.out.println(messages.getString("GMRKeyGeneratorPrimKand"));
	    BigInteger prim = new BigInteger(k / 2, 30, r);
	    /*  p = 7 mod 8 bilden, p != q */
	    
	    if((privKey.p == null)
	       && (prim.mod(BigInteger.valueOf(8)).compareTo(BigInteger.valueOf(7)) == 0)){
		if(privKey.q != null){
		    if(privKey.q.compareTo(prim) != 0){
			System.out.println(messages.getString("GMRKeyGeneratorPFound"));
			privKey.p = prim;
		    }
		}else{
		    System.out.println(messages.getString("GMRKeyGeneratorPFound"));
		    privKey.p = prim;
		}
	    }
	    /* q = 3 mod 8 bilden, p != q */
	    
	    if((privKey.q == null)
	       && (prim.mod(BigInteger.valueOf(8)).compareTo(BigInteger.valueOf(3)) == 0)){
		if(privKey.p != null){
		    if(privKey.p.compareTo(prim) != 0){
			System.out.println(messages.getString("GMRKeyGeneratorQFound"));
			privKey.q = prim;
		    }
		}else{
		    System.out.println(messages.getString("GMRKeyGeneratorQFound"));
		    privKey.q = prim;
		}
	    }
	}
	/* invpq = p^-1 mod q */
	System.out.println(messages.getString("GMRKeyGeneratorCreateinvpq"));
	privKey.invpq = privKey.p.modInverse(privKey.q);
	/* n = p*q bilden */
	
	System.out.println(messages.getString("GMRKeyGeneratorCreate")+" n");
	pubKey.n = privKey.p.multiply(privKey.q);
	privKey.n = pubKey.n;
	/* n/2 bilden */
	
	System.out.println(messages.getString("GMRKeyGeneratorCreate")+" n div2");
	pubKey.ndiv2 = privKey.n.divide(BigInteger.valueOf(2));
	/* (p+1) / 4 */
	System.out.println(messages.getString("GMRKeyGeneratorCreate")+" expp");
	privKey.expp = privKey.p.add(BigInteger.ONE).divide(BigInteger.valueOf(4));
	/* (q+1) / 4 */
	System.out.println(messages.getString("GMRKeyGeneratorCreate")+" expq");
	privKey.expq = privKey.q.add(BigInteger.ONE).divide(BigInteger.valueOf(4));
	/* 4^-1 mod p */
	System.out.println(messages.getString("GMRKeyGeneratorCreate")+" inv4p");
	privKey.inv4p = BigInteger.valueOf(4).modInverse(privKey.p);
	/* 4^-1 mod q */
	System.out.println(messages.getString("GMRKeyGeneratorCreate")+" inv4q");
	privKey.inv4q = BigInteger.valueOf(4).modInverse(privKey.q);

	System.out.println(messages.getString("GMRKeyGeneratorGenEnd"));
    }

    /**
     * Diese Methode gibt den geheimen Schlüssel zurück.
     * @return den geheimen Schlüssel
     */
    
    public GMRPrivateKey getPrivateKey(){
	return privKey;
    }
    
    /**
     * Diese Methode gibt den öffentlichen Schlüssel zurück.
     * @return den öffentlichen Schlüssel
     */
    
    public GMRPublicKey getPublicKey(){
	return pubKey;
    }

    /**
     * Diese Methode generiert eine Referenz zu einem öffentlichen 
     * GMR-Schlüssel.
     *
     * @param aPubKey der öffentliche GMR-Schlüssel
     * @param k gewünschte Bit-Länge der Referenz
     * @return die Referenz
     */
    
    public static BigInteger generateReference(GMRPublicKey aPubKey, int k){
	System.out.println(messages.getString("GMRKeyGeneratorGenRef"));
	BigInteger r = new BigInteger(k,new Random()).modPow(BigInteger.valueOf(2),aPubKey.n);
	if(r.compareTo(aPubKey.ndiv2) > 0 ){
	    return r.subtract(aPubKey.n);
	}
	System.out.println(messages.getString("GMRKeyGeneratorGenR")); 
	return r;
    }

}
